import json
import logging
import sys
from datetime import timedelta

from django.conf import settings
from django.utils import timezone

from apps.nbworker.utils import WorkerState
from apps.notebooks.models import Notebook
from apps.ws.models import Worker
from apps.ws.utils import machine_uuid

log = logging.getLogger(__name__)


class DBClient:
    def __init__(self, notebook_id, session_id, worker_id):
        self.notebook_id = notebook_id
        self.session_id = session_id
        self.worker_id = worker_id
        self.worker = None  # db object
        self.state = WorkerState.Unknown
        self.notebook = None
        self.load_notebook()

    def load_notebook(self):
        try:
            log.debug(f"Load notebook id={self.notebook_id}")
            self.notebook = Notebook.objects.get(pk=self.notebook_id)
        except Exception:
            log.exception("Expetion when notebook load, quit")
            sys.exit(0)

    def is_presentation(self):
        try:
            log.debug(f"Check if notebook is presentation {self.notebook.output}")
            return self.notebook.output == "slides"
        except Exception:
            log.exception("Exception when check if notebook is presentation")
        return False

    def show_code(self):
        try:
            show_it = str(
                json.loads(self.notebook.params).get("show-code", "false")
            ).lower()
            log.debug(f"Check if show code from notebook ({show_it})")
            return show_it == "true"
        except Exception:
            log.exception("Exception when check if show code from notebook")
        return False

    def show_prompt(self):
        try:
            show_it = str(
                json.loads(self.notebook.params).get("show-prompt", "false")
            ).lower()
            log.debug(f"Check if show prompt from notebook ({show_it})")
            return show_it == "true"
        except Exception:
            log.exception("Exception when check if show promtp from notebook")
        return False

    def reveal_theme(self):
        # TODO: get reveal theme
        return "white"

    def worker_state(self):
        return self.state

    def set_worker_state(self, new_state):
        try:
            log.debug(
                f"Worker id={self.worker_id} set state {new_state} uuid {machine_uuid()}"
            )
            self.state = new_state
            if self.worker_exists() and self.worker is not None:
                self.worker.state = new_state
                # set worker machine id
                # to control number of workers
                # in the single machine
                self.worker.machine_id = machine_uuid()
                self.worker.save()
        except Exception:
            log.exception("Exception when set worker state")

    @staticmethod
    def delete_worker_in_db(worker_id):
        try:
            log.debug(f"Delete worker id={worker_id}")
            Worker.objects.get(pk=worker_id).delete()
        except Exception:
            pass
            #log.exception(f"Exception when delete worker")

    def delete_worker(self):
        DBClient.delete_worker_in_db(self.worker_id)

    def delete_stale_workers(self):
        try:
            log.debug(
                (
                    "Delete stale workers, "
                    f"notebook_id={self.notebook_id}, "
                    f"session_id={self.session_id}, "
                    f"worker_id<{self.worker_id}"
                )
            )
            workers = Worker.objects.filter(
                session_id=self.session_id,
                notebook__id=self.notebook_id,
                pk__lt=self.worker_id,
            )
            workers.delete()
        except Exception as e:
            log.exception("Exception when delete stale workers")

    def worker_exists(self):
        try:
            log.debug(f"Worker id={self.worker_id} exists")
            self.worker = Worker.objects.get(pk=self.worker_id)
        except Worker.DoesNotExist as e:
            #log.exception(f"Worker id={self.worker_id} does not exists, quit")
            sys.exit(1)
        return True

    def is_worker_stale(self):
        try:
            log.debug(f"Check worker id={self.worker_id} is stale")
            self.worker = Worker.objects.get(pk=self.worker_id)
            return self.worker.updated_at < timezone.now() - timedelta(minutes=settings.WORKER_STALE_TIME)
           
        except Exception:
            log.exception(
                f"Exception when check if worker id={self.worker_id} is stale"
            )
        return True



import json

import ipywidgets
from IPython.display import display

from .manager import WidgetsManager


class Checkbox:
    def __init__(self, value=True, label=""):
        self.code_uid = WidgetsManager.get_code_uid("Checkbox")
        if WidgetsManager.widget_exists(self.code_uid):
            self.checkbox = WidgetsManager.get_widget(self.code_uid)
            self.checkbox.description = label
        else:
            self.checkbox = ipywidgets.Checkbox(
                value=value, description=label, style={"description_width": "initial"}
            )
            WidgetsManager.add_widget(
                self.checkbox.model_id, self.code_uid, self.checkbox
            )
        display(self)

    @property
    def value(self):
        return self.checkbox.value

    @value.setter
    def value(self, v):
        self.checkbox.value = v

    def __str__(self):
        return "mercury.Checkbox"

    def __repr__(self):

        return "mercury.Checkbox"

    def _repr_mimebundle_(self, **kwargs):
        # data = {}
        # data["text/plain"] = repr(self)
        # return data
        data = self.checkbox._repr_mimebundle_()

        if len(data) > 1:
            view = {
                "widget": "Checkbox",
                "value": self.checkbox.value,
                "label": self.checkbox.description,
                "model_id": self.checkbox.model_id,
                "code_uid": self.code_uid,
            }
            data["application/mercury+json"] = json.dumps(view, indent=4)
            if "text/plain" in data:
                del data["text/plain"]

            return data



import json
import os
import tempfile

import yaml as yaml_lib
from django.contrib.auth.models import User
from django.core import mail
from django.test import TestCase
from rest_framework.reverse import reverse

from apps.notebooks.models import Notebook
from apps.notebooks.tasks import task_init_notebook
from apps.notebooks.tests import create_notebook_with_yaml
from apps.tasks.models import Task
from apps.tasks.notify import notify
from apps.tasks.tasks import sanitize_string, task_execute

# python manage.py test apps.tasks.tests -v 2


class SanitizeTestCase(TestCase):
    def test_CJK(self):

        input_string = """テスト(){}
                        asdfasdf"()"[]''{}:"""

        output_string = sanitize_string(input_string)

        self.assertTrue("(" not in output_string)
        self.assertTrue("[" not in output_string)
        self.assertTrue(output_string.startswith("テスト"))
        self.assertTrue(output_string.endswith("asdfasdf:"))


class ExecuteNotebookTestCase(TestCase):
    def setUp(self):

        task_init_notebook(
            "apps/notebooks/fixtures/third_notebook.ipynb", render_html=False
        )

    def test_task_execute_notebook(self):

        Task.objects.create(notebook_id=1, session_id="test")

        job_params = {
            "db_id": 1,
        }

        task_execute(job_params)


class ExecuteNotebookAuthorizationTestCase(TestCase):
    def test_check_private(self):
        yaml = """---
share: private
---"""
        with tempfile.NamedTemporaryFile() as tmp:
            create_notebook_with_yaml(tmp.name + ".ipynb", yaml=yaml)
            task_init_notebook(tmp.name + ".ipynb")

        # create user and login
        user = {
            "username": "piotrek",
            "password": "verysecret",
        }
        User.objects.create_user(username=user["username"], password=user["password"])
        response = self.client.post(
            reverse("rest_login"), user, content_type="application/json"
        )
        token = response.json()["key"]
        headers = {"HTTP_AUTHORIZATION": "Token " + token}

        # try to execute the notebook as anonymous user
        params = {"session_id": "some_session_id", "params": {}}
        response = self.client.post("/api/v1/execute/1", params)
        self.assertEqual(response.status_code, 404)

        # try to execute the notebook as logged user
        params = {"session_id": "some_session_id", "params": {}}
        response = self.client.post("/api/v1/execute/1", params, **headers)
        self.assertEqual(response.status_code, 201)

        task = Task.objects.get(pk=1)
        self.assertEqual(task.notebook.id, 1)


class RunNotebookRestAPITestCase(TestCase):
    def test_run(self):
        yaml = """---
title: test2
output: rest api
slug: test1
---"""
        with tempfile.NamedTemporaryFile() as tmp:
            create_notebook_with_yaml(tmp.name + ".ipynb", yaml=yaml)
            task_init_notebook(tmp.name + ".ipynb")

        # execute the notebook
        params = {"a": "b", "c": "d"}
        response = self.client.post(
            "/run/test1", params, content_type="application/json"
        )
        self.assertEqual(response.status_code, 201)

        task = Task.objects.get(pk=1)
        self.assertEqual(task.notebook.id, 1)

    def test_run_wrong_slug(self):
        yaml = """---
title: test2
output: rest api
slug: test1
---"""
        with tempfile.NamedTemporaryFile() as tmp:
            create_notebook_with_yaml(tmp.name + ".ipynb", yaml=yaml)
            task_init_notebook(tmp.name + ".ipynb")

        # execute the notebook
        params = {"a": "b", "c": "d"}
        response = self.client.post(
            "/run/test123", params, content_type="application/json"
        )
        self.assertEqual(response.status_code, 404)

    def test_run_not_authenticated(self):
        yaml = """---
title: test2
output: rest api
slug: test1
share: private
---"""
        with tempfile.NamedTemporaryFile() as tmp:
            create_notebook_with_yaml(tmp.name + ".ipynb", yaml=yaml)
            task_init_notebook(tmp.name + ".ipynb")

        # execute the notebook
        params = {"a": "b", "c": "d"}
        response = self.client.post(
            "/run/test1", params, content_type="application/json"
        )
        self.assertEqual(response.status_code, 404)

    def test_run_authenticated(self):
        yaml = """---
title: test2
output: rest api
slug: test1
share: private
---"""
        with tempfile.NamedTemporaryFile() as tmp:
            create_notebook_with_yaml(tmp.name + ".ipynb", yaml=yaml)
            task_init_notebook(tmp.name + ".ipynb")

        # create user and login
        user = {
            "username": "piotrek",
            "password": "verysecret",
        }
        User.objects.create_user(username=user["username"], password=user["password"])
        response = self.client.post(
            reverse("rest_login"), user, content_type="application/json"
        )
        token = response.json()["key"]
        headers = {"HTTP_AUTHORIZATION": "Token " + token}

        # execute the notebook
        params = {"a": "b", "c": "d"}
        response = self.client.post(
            "/run/test1", params, content_type="application/json", **headers
        )
        self.assertEqual(response.status_code, 201)


class GetTaskRestAPITestCase(TestCase):
    def test_get(self):
        yaml = """---
title: test2
output: rest api
slug: test1
---"""
        with tempfile.NamedTemporaryFile() as tmp:
            create_notebook_with_yaml(tmp.name + ".ipynb", yaml=yaml)
            task_init_notebook(tmp.name + ".ipynb")

        session_id = "my_session_id"
        Task.objects.create(
            state="CREATED", session_id=session_id, notebook=Notebook.objects.get(pk=1)
        )

        response = self.client.get(f"/get/{session_id}")
        self.assertEqual(response.status_code, 202)

    def test_get_wrong_session_id(self):
        yaml = """---
title: test2
output: rest api
slug: test1
---"""
        with tempfile.NamedTemporaryFile() as tmp:
            create_notebook_with_yaml(tmp.name + ".ipynb", yaml=yaml)
            task_init_notebook(tmp.name + ".ipynb")

        session_id = "my_session_id"
        Task.objects.create(session_id=session_id, notebook=Notebook.objects.get(pk=1))

        response = self.client.get(f"/get/{session_id}-wrong-id")
        self.assertEqual(response.status_code, 404)

    def test_get_and_run(self):
        yaml = """---
title: test2
output: rest api
slug: test1
params:
    rest_response:
        output: response
---"""
        code = "rest_response = 'example.json'"

        code2 = """
import os
with open(rest_response, "w") as fout:
    fout.write('{"hello": "world"}')        
"""
        with tempfile.NamedTemporaryFile() as tmp:
            create_notebook_with_yaml(
                tmp.name + ".ipynb", yaml=yaml, code=code, code2=code2
            )
            task_init_notebook(tmp.name + ".ipynb")

        params = {"a": "b", "c": "d"}
        response = self.client.post(
            "/run/test1", params, content_type="application/json"
        )
        self.assertEqual(response.status_code, 201)

        task = Task.objects.get(pk=1)
        self.assertEqual(task.notebook.id, 1)

        response = self.client.get(f"/get/{task.session_id}")
        self.assertEqual(response.status_code, 202)
        self.assertEqual(response.json()["state"], "running")

        task_execute(job_params={"db_id": 1})

        response = self.client.get(f"/get/{task.session_id}")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), {"hello": "world"})


class NotifyTestCase(TestCase):
    def test_notify(self):
        User.objects.create_user(
            username="username1", password="password", email="contact@example.com"
        )
        config = """---
title: my notebook
schedule: '* * * * *'
notify:
    on_success: username1, contact@example.com
    attachment: html   
---"""

        with tempfile.NamedTemporaryFile() as tmp:

            create_notebook_with_yaml(
                tmp.name + ".ipynb", yaml=config, code="print('hello')"
            )
            task_init_notebook(tmp.name + ".ipynb")

            Task.objects.create(notebook_id=1, session_id="test")
            job_params = {"db_id": 1}
            task_execute(job_params)

            notebook = Notebook.objects.get(pk=1)
            task = Task.objects.get(pk=1)

            # notify(json.loads(notebook.notify), True, "", notebook.id, task.result)

        print(mail.outbox[0].subject)
        # print(mail.outbox[1].subject)
        # Test that one message has been sent.
        self.assertEqual(len(mail.outbox), 1)

        # Verify that the subject of the first message is correct.
        self.assertTrue("success" in mail.outbox[0].subject)



from django.conf.urls import url

from apps.notebooks.views import ListNotebooks, RetrieveNotebook

notebooks_urlpatterns = [
    url("api/v1/notebooks/(?P<notebook_id>.+)", RetrieveNotebook.as_view()),
    url("api/v1/notebooks", ListNotebooks.as_view()),
]



import json

from IPython.display import display

from .manager import WidgetsManager


class App:
    def __init__(
        self,
        title="",
        description="",
        show_code=False,
        show_prompt=False,
        share="public",
        output="app",
        slug="",
        schedule="",
        notify={},
        continuous_update=True,
        static_notebook=False,
    ):
        self.code_uid = WidgetsManager.get_code_uid("App")
        self.title = title
        self.description = description
        self.show_code = show_code
        self.show_prompt = show_prompt
        self.share = share
        self.output = output
        self.slug = slug
        self.schedule = schedule
        self.notify = notify
        self.continuous_update = continuous_update
        self.static_notebook = static_notebook
        display(self)

    def __repr__(self):
        return f"mercury.App"

    def _repr_mimebundle_(self, **kwargs):
        data = {}
        data["text/plain"] = repr(self)
        data["text/html"] = "<h3>Mercury Application</h3><small>This output won't appear in the web app.</small>"
        view = {
            "widget": "App",
            "title": self.title,
            "description": self.description,
            "show_code": self.show_code,
            "show_prompt": self.show_prompt,
            "share": self.share,
            "output": self.output,
            "slug": self.slug,
            "schedule": self.schedule,
            "notify": json.dumps(self.notify),
            "continuous_update": self.continuous_update,
            "static_notebook": self.static_notebook,
            "model_id": "mercury-app",
            "code_uid": self.code_uid,
        }
        data["application/mercury+json"] = json.dumps(view, indent=4)
        return data



import os

from django.core.management.base import BaseCommand, CommandError

from apps.notebooks.models import Notebook
from apps.notebooks.tasks import task_init_notebook


class Command(BaseCommand):
    help = "Add a new notebook"

    def add_arguments(self, parser):
        parser.add_argument("notebook_path", help="Path to notebook")

    def handle(self, *args, **options):
        notebook_path = options["notebook_path"]
        notebook_id = self.notebook_id_available(notebook_path)

        action = "added"
        if notebook_id is None:
            self.stdout.write(self.style.HTTP_INFO(f"Initialize {notebook_path}"))
        else:
            action = "updated"
            self.stdout.write(
                self.style.HTTP_INFO(f"The notebook {notebook_path} will be updated")
            )

        notebook_id = task_init_notebook(
            options["notebook_path"], notebook_id=notebook_id
        )

        self.stdout.write(
            self.style.SUCCESS(f"Successfully {action} a notebook (id:{notebook_id})")
        )

    def notebook_id_available(self, notebook_path):
        notebook = Notebook.objects.filter(path=os.path.abspath(notebook_path)).first()
        if notebook is not None:
            return notebook.id
        return None



import copy
import json
import logging
import sys
import threading
import time
import os
import hashlib

from datetime import datetime
from django_drf_filepond.models import TemporaryUpload
from execnb.nbio import nb2dict, read_nb

from apps.nb.nbrun import NbRun
from apps.nbworker.utils import Purpose, WorkerState
from apps.nbworker.ws import WSClient
from apps.storage.storage import StorageManager
from apps.tasks.export_pdf import to_pdf
from apps.tasks.models import Task
from apps.ws.utils import parse_params
from apps.nbworker.utils import stop_event
from widgets.manager import WidgetsManager

log = logging.getLogger(__name__)


class NBWorker(WSClient):
    def __init__(self, ws_address, notebook_id, session_id, worker_id):

        super(NBWorker, self).__init__(ws_address, notebook_id, session_id, worker_id)

        self.prev_nb = None
        self.prev_widgets = {}
        self.prev_body = ""
        self.prev_update_time = None
        self.prev_md5 = None
        # monitor notebook file updates if running locally
        if "127.0.0.1" in ws_address:
            threading.Thread(target=self.nb_file_watch, daemon=True).start()
        threading.Thread(target=self.process_msgs, daemon=True).start()
        self.ws.run_forever(ping_interval=5, ping_timeout=3)

    @staticmethod
    def md5(fname):
        hash_md5 = hashlib.md5()
        with open(fname, "rb") as f:
            for chunk in iter(lambda: f.read(4096), b""):
                hash_md5.update(chunk)
        return hash_md5.hexdigest()

    def nb_file_watch(self):
        while not stop_event.is_set():
            current_update_time = datetime.fromtimestamp(
                os.path.getmtime(self.notebook.path)
            )
            if (
                self.prev_update_time is not None
                and self.prev_update_time != current_update_time
            ):
                checksum = NBWorker.md5(self.notebook.path)

                log.debug(f"Checksum {checksum} prev {self.prev_md5}")

                if self.prev_md5 is None or checksum != self.prev_md5:

                    log.debug("Notebook file changed!")
                    msg = json.dumps({"purpose": Purpose.InitNotebook})
                    self.queue.put(msg)

                self.prev_md5 = checksum

            self.prev_update_time = current_update_time
            time.sleep(0.25)

    def process_msgs(self):
        global stop_event
        while not stop_event.is_set():
            item = self.queue.get()
            log.debug(f"Porcess msg {item}")
            json_data = json.loads(item)

            if json_data.get("purpose", "") == Purpose.InitNotebook:
                self.init_notebook()
            elif json_data.get("purpose", "") == Purpose.RunNotebook:
                self.run_notebook(json_data)
            elif json_data.get("purpose", "") == Purpose.SaveNotebook:
                self.save_notebook()
            elif json_data.get("purpose", "") == Purpose.DisplayNotebook:
                self.display_notebook(json_data)
            elif json_data.get("purpose", "") == Purpose.ClearSession:
                self.init_notebook()
            elif json_data.get("purpose", "") == Purpose.WorkerPing:
                self.worker_pong()
            elif json_data.get("purpose", "") == Purpose.CloseWorker:
                stop_event.set()
                self.delete_worker()
                sys.exit(1)
            elif json_data.get("purpose", "") == Purpose.DownloadHTML:
                self.download_html()
            elif json_data.get("purpose", "") == Purpose.DownloadPDF:
                self.download_pdf()

            self.queue.task_done()

    def worker_pong(self):
        self.update_worker_state(WorkerState.Running)
        if self.worker_exists():
            self.send_state()

    def run_notebook(self, json_params):
        log.debug(f"Run notebook with {json_params}")
        self.update_worker_state(WorkerState.Busy)

        widgets = json.loads(json_params.get("widgets", "{}"))

        self.update_nb(widgets)

        if self.is_presentation():
            body = self.nbrun.export_html(self.nb, full_header=True)
        else:
            body = self.nbrun.export_html(self.nb, full_header=False)

        # with open(f"test_{counter}.html", "w") as fout:
        #    fout.write(body)

        self.ws.send(json.dumps({"purpose": Purpose.ExecutedNotebook, "body": body}))
        self.update_worker_state(WorkerState.Running)
        self.prev_body = copy.deepcopy(body)

    def update_nb(self, widgets):
        log.debug(f"Update nb {widgets}")

        index_execute_from = 1
        # fill notebook with widgets values
        self.prev_widgets = copy.deepcopy(widgets)
        for widget_key in widgets.keys():
            value = widgets[widget_key]

            widget_type = WidgetsManager.parse_widget_type(widget_key)
            log.debug(
                f"Update widget code_uid={widget_key} value={value} widget type {widget_type}"
            )

            if widget_type == "File" and len(value) == 2:
                log.debug(f"Get file {value[0]} from id={value[1]}")
                tu = TemporaryUpload.objects.get(upload_id=value[1])
                value[1] = tu.get_file_path()
                log.debug(f"File path is {value[1]}")

                code = (
                    f'WidgetsManager.update("{widget_key}", field="filename", new_value="{value[0]}")\n'
                    f'WidgetsManager.update("{widget_key}", field="filepath", new_value="{value[1]}")\n'
                )
            elif widget_type == "OutputDir":

                sm = StorageManager(self.session_id, self.worker_id)
                output_dir = sm.worker_output_dir()
                code = f'WidgetsManager.update("{widget_key}", field="value", new_value="{output_dir}")'

            elif isinstance(value, str):
                code = f'WidgetsManager.update("{widget_key}", field="value", new_value="{value}")'
            else:
                code = f'WidgetsManager.update("{widget_key}", field="value", new_value={value})'

            log.debug(f"Execute code {code}")

            r = self.nbrun.run_code(code)

            updated = "True" in str(r)
            log.debug(f"Update reponse {r}, updated={updated}")

            if updated:
                cell_index = WidgetsManager.parse_cell_index(widget_key)
                log.debug(f"Widget updated, update nb from {cell_index}")

                if index_execute_from == 1:
                    index_execute_from = cell_index
                else:
                    index_execute_from = min(index_execute_from, cell_index)

        if index_execute_from != 1:
            if self.prev_nb is not None:
                self.nb = copy.deepcopy(self.prev_nb)
            else:
                self.nb = copy.deepcopy(self.nb_original)

            for i in range(index_execute_from, len(self.nb.cells) + 1):

                log.debug(f"Execute cell index={i-1}")

                self.nbrun.run_cell(self.nb.cells[i - 1], counter=i)

            """
                print(self.nb.cells[i - 1])

                for output in self.nb.cells[i - 1].get("outputs", []):
                    if "data" in output:
                        if "application/mercury+json" in output["data"]:
                            w = output["data"]["application/mercury+json"]
                            log.debug(w)
                            w = json.loads(w)

                            # prepare msg to send by ws
                            msg = WidgetsManager.frontend_format(w)
                            if msg:
                                msg["purpose"] = Purpose.UpdateWidgets
                                msg["widgetKey"] = w.get("code_uid")
                                log.debug(f"Update widget {msg}")
                                self.ws.send(json.dumps(msg))

            # check if hide some widgets
            nb_widgets_keys = []
            for cell in self.nb.cells:
                for output in cell.get("outputs", []):
                    w = output.get("data", {}).get("application/mercury+json")
                    if w is not None:
                        w = json.loads(w)
                        code_uid = w.get("code_uid")
                        if code_uid is not None:
                            nb_widgets_keys += [code_uid]

            hide_widgets = []
            for widget_key in widgets.keys():
                if widget_key not in nb_widgets_keys:
                    hide_widgets += [widget_key]

            log.debug(f"Hide widgets {hide_widgets}")
            if hide_widgets:
                msg = {"purpose": Purpose.HideWidgets, "keys": hide_widgets}
                self.ws.send(json.dumps(msg))
            """
            self.send_widgets(self.nb, expected_widgets_keys=widgets.keys())
            self.prev_nb = copy.deepcopy(self.nb)
        else:
            log.debug("Skip nb execution, no changes in widgets")

    def send_widgets(self, nb, expected_widgets_keys, init_widgets=False):
        nb_widgets_keys = []
        widgets_params = []
        for cell in nb.cells:
            for output in cell.get("outputs", []):
                if "data" in output:
                    if "application/mercury+json" in output["data"]:
                        w = output["data"]["application/mercury+json"]
                        log.debug(w)
                        w = json.loads(w)

                        # prepare msg to send by ws
                        msg = WidgetsManager.frontend_format(w)
                        if msg:
                            msg["widgetKey"] = w.get("code_uid")
                            log.debug(f"Update widget {msg}")
                            if init_widgets:
                                widgets_params += [msg]
                            else:
                                msg["purpose"] = Purpose.UpdateWidgets
                                self.ws.send(json.dumps(msg))

                        code_uid = w.get("code_uid")
                        if code_uid is not None:
                            nb_widgets_keys += [code_uid]

        if init_widgets:
            msg = {"purpose": Purpose.InitWidgets, "widgets": widgets_params}
            log.debug("------------Init widgets")
            log.debug(msg)
            self.ws.send(json.dumps(msg))
        else:
            # check if hide some widgets
            # needed only when updating widgets
            hide_widgets = []
            for widget_key in expected_widgets_keys:
                if widget_key not in nb_widgets_keys:
                    hide_widgets += [widget_key]

            log.debug(f"Hide widgets {hide_widgets}")
            if hide_widgets:
                msg = {"purpose": Purpose.HideWidgets, "keys": hide_widgets}
                self.ws.send(json.dumps(msg))

    def init_notebook(self):
        log.debug(f"Init notebook, show_code={self.show_code()}")

        self.prev_nb = None
        self.prev_widgets = {}
        self.prev_body = ""

        self.update_worker_state(WorkerState.Busy)

        self.nbrun = NbRun(
            show_code=self.show_code(),
            show_prompt=self.show_prompt(),
            is_presentation=self.is_presentation(),
            reveal_theme=self.reveal_theme(),
        )
        self.nb_original = read_nb(self.notebook.path)

        self.nbrun.run_notebook(self.nb_original)

        # TODO: update params in db if needed"
        params = {}
        parse_params(nb2dict(self.nb_original), params)

        # update database ...
        log.debug(f"Executed params {json.dumps(params, indent=4)}")

        update_database = False
        if self.notebook.title != params.get("title", ""):
            self.notebook.title = params.get("title", "")
            update_database = True

        nb_params = json.loads(self.notebook.params)
        for property in [
            "show-code",
            "show-prompt",
            "continuous_update",
            "static_notebook",
            "description"
        ]:
            if params.get(property) is not None and nb_params.get(
                property
            ) != params.get(property):
                nb_params[property] = params.get(property)
                update_database = True
        # save widgets params
        if json.dumps(nb_params.get("params", {})) != json.dumps(params.get("params", {})):
            nb_params["params"] = params["params"]
            update_database = True

        if update_database:
            self.notebook.params = json.dumps(nb_params)
            self.notebook.save()

        self.nbrun.set_show_code_and_prompt(
            nb_params.get("show-code", False), nb_params.get("show-prompt", True)
        )
        self.nbrun.set_is_presentation(nb_params.get("output", "app") == "slides")

        log.debug(params)
        log.debug(f"Exporter show_code {self.nbrun.exporter.show_code}")

        self.nb = copy.deepcopy(self.nb_original)

        if self.is_presentation():
            body = self.nbrun.export_html(self.nb, full_header=True)
        else:
            body = self.nbrun.export_html(self.nb, full_header=False)

        msg = {"purpose": Purpose.ExecutedNotebook, "body": body}
        if update_database:
            msg["reloadNotebook"] = True
        self.ws.send(json.dumps(msg))
        self.prev_body = copy.deepcopy(body)

        self.send_widgets(self.nb, expected_widgets_keys=[], init_widgets=True)
        self.update_worker_state(WorkerState.Running)

    def save_notebook(self):
        log.debug(f"Save notebook")
        # save nb in HTML
        if self.is_presentation():
            nb_body = self.nbrun.export_html(self.nb, full_header=True)
        else:
            nb_body = self.nbrun.export_html(self.nb, full_header=True)

        sm = StorageManager(self.session_id, self.worker_id)
        nb_path = sm.save_nb_html(nb_body)

        # create task with path to HTML file
        task = Task.objects.create(
            task_id=f"worker-{self.worker_id}",
            session_id=self.session_id,
            notebook_id=self.notebook_id,
            state="DONE",
            params=json.dumps(self.prev_widgets),
            result=nb_path,
        )
        log.debug(f"Task ({task.id}) created")

        # send notice that nb saved
        self.ws.send(json.dumps({"purpose": Purpose.SavedNotebook}))

    def display_notebook(self, json_params):
        log.debug(f"Display notebook ({json_params})")

    def download_html(self):
        log.debug(f"Download HTML")
        # save nb in HTML with full header
        if self.is_presentation():
            nb_body = self.nbrun.export_html(self.nb, full_header=True)
        else:
            nb_body = self.nbrun.export_html(self.nb, full_header=True)

        sm = StorageManager(self.session_id, self.worker_id)
        _, nb_url = sm.save_nb_html(nb_body)

        # send HTML url address
        self.ws.send(
            json.dumps(
                {
                    "purpose": Purpose.DownloadHTML,
                    "url": nb_url,
                    "filename": f"{self.notebook.slug}.html",
                }
            )
        )

    def download_pdf(self):
        log.debug(f"Download PDF")
        # save nb in HTML with full header
        if self.is_presentation():
            nb_body = self.nbrun.export_html(self.nb, full_header=True)
        else:
            nb_body = self.nbrun.export_html(self.nb, full_header=True)

        # export to PDF
        sm = StorageManager(self.session_id, self.worker_id)
        _, pdf_url = sm.save_nb_pdf(nb_body, self.is_presentation())

        # send PDF url
        self.ws.send(
            json.dumps(
                {
                    "purpose": Purpose.DownloadPDF,
                    "url": pdf_url,
                    "filename": f"{self.notebook.slug}.pdf",
                }
            )
        )



import logging
import os
import shutil
import uuid

from django.conf import settings
from execnb.nbio import write_nb

from apps.tasks.export_pdf import to_pdf

log = logging.getLogger(__name__)


class StorageManager:
    def __init__(self, session_id, worker_id):
        self.session_id = session_id
        self.worker_id = worker_id

    @staticmethod
    def create_dir(dir_path):
        if not os.path.exists(dir_path):
            try:
                log.debug(f"Create directory {dir_path}")
                os.mkdir(dir_path)
            except Exception as e:
                log.exception(f"Exception when create {dir_path}")
                raise Exception(f"Cant create {dir_path}")
        else:
            log.debug(f"Directory {dir_path} already exists")

    @staticmethod
    def delete_dir(dir_path):
        if os.path.exists(dir_path):
            try:
                log.debug(f"Delete directory {dir_path}")
                shutil.rmtree(dir_path, ignore_errors=True)
            except Exception as e:
                log.exception(f"Exception when delete {dir_path}")
                raise Exception(f"Cant delete {dir_path}")

    def worker_output_dir(self):
        if settings.STORAGE == settings.STORAGE_MEDIA:
            first_dir = os.path.join(str(settings.MEDIA_ROOT), self.session_id)
            StorageManager.create_dir(first_dir)
            output_dir = os.path.join(
                str(settings.MEDIA_ROOT), self.session_id, f"output_{self.worker_id}"
            )
            StorageManager.create_dir(output_dir)
            log.debug(f"Worker output directory: {output_dir}")
            return output_dir
        raise Exception("Other methods for worker output directory are not implemented")

    def delete_worker_output_dir(self):
        if settings.STORAGE == settings.STORAGE_MEDIA:
            output_dir = self.worker_output_dir(self.session_id, self.worker_id)
            StorageManager.delete_dir(output_dir)
            log.debug(f"Deleted worker output directory: {output_dir}")
            return output_dir
        raise Exception(
            "Other methods to delete worker output directory are not implemented"
        )

    def list_worker_files_urls(self):
        files_urls = []
        if settings.STORAGE == settings.STORAGE_MEDIA:
            output_dir = self.worker_output_dir()
            for f in os.listdir(output_dir):
                if os.path.isfile(os.path.join(output_dir, f)):
                    files_urls += [
                        f"{settings.MEDIA_URL}/{self.session_id}/output_{self.worker_id}/{f}"
                    ]
        return files_urls

    def save_nb(self, nb):
        fpath = None
        if settings.STORAGE == settings.STORAGE_MEDIA:
            fpath = os.path.join(
                self.worker_output_dir(), f"nb-{self.some_hash()}.ipynb"
            )
            write_nb(nb, fpath)
        return fpath

    def save_nb_html(self, nb_html_body):
        fpath, url = None, None
        if settings.STORAGE == settings.STORAGE_MEDIA:
            fname = f"download-notebook-{self.some_hash()}.html"
            fpath = os.path.join(self.worker_output_dir(), fname)
            with open(fpath, "w", encoding="utf-8", errors="ignore") as fout:
                fout.write(nb_html_body)
            url = f"{settings.MEDIA_URL}/{self.session_id}/output_{self.worker_id}/{fname}"
        return fpath, url

    def save_nb_pdf(self, nb_html_body, is_presentation):
        pdf_path, pdf_url = None, None
        if settings.STORAGE == settings.STORAGE_MEDIA:
            # save HTML
            html_path, html_url = self.save_nb_html(nb_html_body)

            # check if we need postfix
            slides_postfix = ""
            if is_presentation == "slides":
                slides_postfix = "?print-pdf"

            # export to PDF
            pdf_path = html_path.replace(".html", ".pdf")
            pdf_url = html_url.replace(".html", ".pdf")
            to_pdf(f"{html_path}{slides_postfix}", pdf_path)

        return pdf_path, pdf_url

    def some_hash(self):
        h = uuid.uuid4().hex.replace("-", "")
        return h[:8]



class SlidesThemes:
    @staticmethod
    def nbconvert_options(format):
        theme = format.get("theme", "white")

        if theme == "black":
            return [
                "--SlidesExporter.reveal_theme=black",
                "--SlidesExporter.theme=dark",
            ]
        elif theme == "white":
            return [
                "--SlidesExporter.reveal_theme=white",
                "--SlidesExporter.theme=light",
            ]
        elif theme == "league":
            return [
                "--SlidesExporter.reveal_theme=league",
                "--SlidesExporter.theme=dark",
            ]
        elif theme == "sky":
            return [
                "--SlidesExporter.reveal_theme=sky",
                "--SlidesExporter.theme=light",
            ]
        elif theme == "beige":
            return [
                "--SlidesExporter.reveal_theme=beige",
                "--SlidesExporter.theme=light",
            ]
        elif theme == "simple":
            return [
                "--SlidesExporter.reveal_theme=simple",
                "--SlidesExporter.theme=light",
            ]
        elif theme == "serif":
            return [
                "--SlidesExporter.reveal_theme=serif",
                "--SlidesExporter.theme=light",
            ]
        elif theme == "blood":
            return [
                "--SlidesExporter.reveal_theme=blood",
                "--SlidesExporter.theme=dark",
            ]
        elif theme == "night":
            return [
                "--SlidesExporter.reveal_theme=night",
                "--SlidesExporter.theme=dark",
            ]
        elif theme == "moon":
            return [
                "--SlidesExporter.reveal_theme=moon",
                "--SlidesExporter.theme=dark",
            ]
        elif theme == "solarized":
            return [
                "--SlidesExporter.reveal_theme=solarized",
                "--SlidesExporter.theme=light",
            ]

        return []

    @staticmethod
    def additional_css(format):
        theme = format.get("theme", "white")

        if theme == "black":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(255, 255, 255);
    font-family: "Source Sans Pro", Helvetica, sans-serif;
}
</style>"""
        elif theme == "white":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(34, 34, 34);
    font-family: "Source Sans Pro", Helvetica, sans-serif;
}
</style>"""
        elif theme == "league":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(238, 238, 238);
    font-family: "Lato", sans-serif;
}
</style>"""
        elif theme == "sky":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(51, 51, 51);
    font-family: "Open Sans", sans-serif;
}
</style>"""
        elif theme == "beige":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(51, 51, 51);
    font-family: "Lato", sans-serif;
}
</style>"""
        elif theme == "simple":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(0, 0, 0);
    font-family: "Lato", sans-serif;
}
</style>"""
        elif theme == "serif":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(0, 0, 0);
    font-family: "Palatino Linotype", "Book Antiqua", Palatino, FreeSerif, serif;
}
</style>"""
        elif theme == "blood":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(238, 238, 238);
    font-family: Ubuntu, "sans-serif";  
}
</style>"""
        elif theme == "night":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(238, 238, 238);
    font-family: "Open Sans", sans-serif;  
}
</style>"""
        elif theme == "moon":
            return """\n<style type="text/css">
.reveal * {
    rgb(147, 161, 161);
    font-family: "Lato", sans-serif;
}
</style>"""
        elif theme == "solarized":
            return """\n<style type="text/css">
.reveal * {
    color: rgb(101, 123, 131);
    font-family: "Lato", sans-serif;
}
</style>"""

        return ""



import json

import ipywidgets
from IPython.display import display

from .manager import WidgetException, WidgetsManager


class Slider:
    def __init__(self, value=0, min=0, max=10, label="", step=1):
        if value < min:
            raise WidgetException("value should be equal or larger than min")
        if value > max:
            raise WidgetException("value should be equal or smaller than max")

        self.code_uid = WidgetsManager.get_code_uid("Slider")

        if WidgetsManager.widget_exists(self.code_uid):
            self.slider = WidgetsManager.get_widget(self.code_uid)
            if self.slider.min != min:
                self.slider.min = min
                self.slider.value = value
            if self.slider.max != max:
                self.slider.max = max
                self.slider.value = value
            if self.slider.step != step:
                self.slider.step = step
                self.slider.value = value
            self.slider.description = label
        else:
            self.slider = ipywidgets.IntSlider(
                value=value,
                min=min,
                max=max,
                description=label,
                step=step,
                style={"description_width": "initial"},
            )
            WidgetsManager.add_widget(self.slider.model_id, self.code_uid, self.slider)
        display(self)

    @property
    def value(self):
        return self.slider.value

    # @value.setter
    # def value(self, v):
    #    self.slider.value = v

    def __str__(self):
        return "mercury.Slider"

    def __repr__(self):

        return "mercury.Slider"

    def _repr_mimebundle_(self, **kwargs):
        # data = {}
        # data["text/plain"] = repr(self)
        # return data
        data = self.slider._repr_mimebundle_()

        if len(data) > 1:
            view = {
                "widget": "Slider",
                "value": self.slider.value,
                "min": self.slider.min,
                "max": self.slider.max,
                "step": self.slider.step,
                "label": self.slider.description,
                "model_id": self.slider.model_id,
                "code_uid": self.code_uid,
            }
            data["application/mercury+json"] = json.dumps(view, indent=4)
            if "text/plain" in data:
                del data["text/plain"]

            return data



from enum import Enum
import threading

stop_event = threading.Event()


class WorkerState(str, Enum):
    Busy = "Busy"
    Running = "Running"
    Unknown = "Unknown"


class Purpose(str, Enum):
    WorkerPing = "worker-ping"
    WorkerState = "worker-state"
    InitNotebook = "init-notebook"
    RunNotebook = "run-notebook"
    SaveNotebook = "save-notebook"
    SavedNotebook = "saved-notebook"
    DisplayNotebook = "display-notebook"
    ClearSession = "clear-session"
    CloseWorker = "close-worker"

    ExecutedNotebook = "executed-notebook"
    UpdateWidgets = "update-widgets"
    HideWidgets = "hide-widgets"
    InitWidgets = "init-widgets"
    
    DownloadHTML = "download-html"
    DownloadPDF = "download-pdf"



from django.contrib import admin

from apps.notebooks.models import Notebook


class NotebookModelAdmin(admin.ModelAdmin):
    list_display = ("id", "title", "slug", "state", "path")

    class Meta:
        model = Notebook


admin.site.register(Notebook, NotebookModelAdmin)



import sys
import os
import tempfile
import nbformat as nbf

def info(package):
    print("To run demo you need to have: pandas, numpy and matplotlib packages")
    print()
    print(f"Cant import {package} package")
    print(f"Please install {package} with command:")
    print()
    print(f"pip install {package}")
    print()
    sys.exit(1)

def check_needed_packages():
    try:
        import pandas
    except Exception as e:
        info("pandas")
    try:
        import numpy
    except Exception as e:
        info("numpy")
    try:
        import matplotlib
    except Exception as e:
        info("matplotlib")


def create_simple_demo_notebook(filename="demo.ipynb"):
    nb = nbf.v4.new_notebook()

    nb["cells"] = [
        nbf.v4.new_code_cell("""import mercury as mr
app = mr.App(title="Hello demo 👋", description="Hello demo with Text widget", show_code=True)
"""),
        nbf.v4.new_markdown_cell(
            """# Mercury demo 👋

Share your notebooks with everyone thanks to Mercury framework!

Please write your name in the left side bar and press **Enter** ⌨️ 

The notebook will be automatically recomputed. Only cells with widget definition and below are recomputed. That's why it is fast!

You can download executed notebook as HTML or PDF (just click in the left side bar).

You can edit this notebook in the Jupyter Notebook, and changes will appear instantly in the Mercury.
"""
        ),
        nbf.v4.new_code_cell(
            """name = mr.Text(label="What is you name?", value="Piotr")"""
        ),
        nbf.v4.new_code_cell(
            '''mr.Markdown(f"""## Hello {name.value}! 

For more examples please check our documentation at <a href="https://runmercury.com" target="_blank">RunMercury.com<a> 📚
""")'''
        ),
    ]
    with open(filename, "w", encoding="utf-8", errors="ignore") as f:
        nbf.write(nb, f)


def create_demo_notebook(filename="demo-dataframe-and-plots.ipynb"):
    nb = nbf.v4.new_notebook()

    nb["cells"] = [
        nbf.v4.new_code_cell(
            """import pandas as pd
import numpy as np
from matplotlib import pyplot as plt"""
        ),
        nbf.v4.new_code_cell("import mercury as mr"),
        nbf.v4.new_code_cell(
            """# control app with App class
app = mr.App(title="DataFrame & Plots 🚀", description="Showcase of Mercury Widgets", show_code = False)"""
        ),
        nbf.v4.new_markdown_cell(
            """# DataFrame and Plots 🎲📊

Share your notebooks with everyone thanks to Mercury framework.

Please change number of samples and number of features in the left side bar. Notebook will be recomputed after widget change."""
        ),
        nbf.v4.new_code_cell(
            """samples = mr.Slider(label="Number of samples", min=50, max=100, value=75)
features = mr.Select(label="Number of features", choices=["5", "10", "15"], value="10")"""
        ),
        nbf.v4.new_code_cell(
            """data = {}
for i in range(int(features.value)):
    data[f"Feature {i}"] = np.random.rand(samples.value)
df = pd.DataFrame(data)"""
        ),
        nbf.v4.new_markdown_cell("## Random data 🎲"),
        nbf.v4.new_code_cell("df"),
        nbf.v4.new_markdown_cell("## Scatter plot 📈"),
        nbf.v4.new_code_cell("""_ = plt.plot(df["Feature 1"], df["Feature 2"], '*')"""),
        nbf.v4.new_markdown_cell("## Random data histogram 📊"),
        nbf.v4.new_code_cell("""_ = plt.hist(df["Feature 1"], bins=40)"""),
    ]
    with open(filename, "w", encoding="utf-8", errors="ignore") as f:
        nbf.write(nb, f)


def create_slides_demo_notebook(filename="demo-slides.ipynb"):
    nb = nbf.v4.new_notebook()

    nb["cells"] = [
        nbf.v4.new_code_cell(
            """import mercury as mr
import numpy as np
from matplotlib import pyplot as plt

app = mr.App(title="Slides demo 📝", description="Wouldn't it be amazing to recompute slides during the show?") """,
            metadata={"slideshow": {"slide_type": "skip"}},
        ),
        nbf.v4.new_markdown_cell(
            """# Interactive presentation 📝""",
            metadata={"slideshow": {"slide_type": "slide"}},
        ),
        nbf.v4.new_markdown_cell(
            """## Recompute slides 🖥️

- You can create interactive presentation with Mercury
- Users can recompute slides by changing widgets
- You can enter full screen by pressing **F** and exit with **Esc**
- Please check next slides ➡️

""",
            metadata={"slideshow": {"slide_type": "slide"}},
        ),
        nbf.v4.new_code_cell(
            """name = mr.Text(label="What is your name?", value="Piotr")""",
            metadata={"slideshow": {"slide_type": "skip"}},
        ),
        nbf.v4.new_code_cell(
            '''mr.Markdown(f"""## Hello {name.value}!

{name.value}, this slide is recomputed after name change in the left side bar.

Please change the name value in the left side bar and press **Enter**.

Please check next slide ➡️

""")''',
            metadata={"slideshow": {"slide_type": "slide"}},
        ),
        nbf.v4.new_code_cell(
            """samples = mr.Slider(label="How many samples", value=75, min=50, max=100)
color = mr.Select(label="Mark color", value="blue", choices=["blue", "green", "red"])
""",
            metadata={"slideshow": {"slide_type": "skip"}},
        ),
        nbf.v4.new_code_cell(
            '''mr.Markdown("""## Scatter plot 🎲
Please change number of samples and mark color in the left side bar. The plot will be updated during the slide show.""")
_ = plt.plot(np.random.rand(samples.value), np.random.rand(samples.value), "*", color=color.value)''',
            metadata={"slideshow": {"slide_type": "slide"}},
        ),
        nbf.v4.new_markdown_cell("""## Thank you!
        
Please check our documentation at <a href="https://runmercury.com" target="_blank">RunMercury.com</a> for more information 📚

""", metadata={"slideshow": {"slide_type": "slide"}})
    ]
    with open(filename, "w", encoding="utf-8", errors="ignore") as f:
        nbf.write(nb, f)


def create_welcome(filename="welcome.md"):
    content = """
# Welcome in Mercury 👋

Mercury framework allows you easily turn Jupyter Notebooks into shareble web applications.

You can create beautiful and interactive web applications, reports, dashboards and presentations.

Mercury features:
- add widgets with simple Python API, 
- simple cell execution model - widgets trigger cell execution below the widget definition,
- hide or show notebook code,
- share multiple notebooks,
- executed notebook can be exported to HTML or PDF,
- embed notebook apps on any website,
- easily deploy (free & public Mercury cloud comming soon!)
- easily add authentication to notebooks (comming soon!)
- schedule automatic execution (comming soon!)

Please check our documentation at <a href="https://runmercury.com" target="_blank">RunMercury.com</a> for more information 📚

This text can be edited by changing `welcome.md` file. Demo notebooks can be edited in Jupyter. 

All files created for demo are in the current directory.

## Demo applications

    """
    with open(filename, "w", encoding="utf-8", errors="ignore") as f:
        f.write(content)



from django.db import transaction
from django.db.models import Q
from django.http import HttpResponse, JsonResponse
from django.shortcuts import get_object_or_404
from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework.views import APIView

from apps.notebooks.models import Notebook
from apps.notebooks.serializers import NotebookSerializer
from apps.notebooks.tasks import task_init_notebook, task_watch
from server.settings import is_pro

if is_pro:
    from pro.accounts.models import Membership


def in_commas(word):
    return "," + word + ","


def notebooks_queryset(request):
    if not is_pro:
        return Notebook.objects.all()

    user = request.user
    if user.is_anonymous:
        return Notebook.objects.filter(share=in_commas("public"))

    q_list = (
        Q(share=in_commas("public"))
        | Q(share=in_commas("private"))
        | Q(share__icontains=in_commas(user.username))
    )

    for m in Membership.objects.filter(user=user):
        q_list |= Q(share__icontains=in_commas(m.group.name))

    return Notebook.objects.filter(q_list)


class ListNotebooks(APIView):
    def get(self, request, format=None):
        notebooks = notebooks_queryset(request)
        serializer = NotebookSerializer(notebooks, many=True)
        return JsonResponse(serializer.data, safe=False)


class RetrieveNotebook(APIView):
    def get(self, request, notebook_id, format=None):
        pk = int(notebook_id.replace("/", ""))
        notebook = get_object_or_404(notebooks_queryset(request), pk=pk)
        serializer = NotebookSerializer(notebook)
        if notebook.state.startswith("WATCH"):
            task_watch.delay(notebook.id)

        return JsonResponse(serializer.data, safe=False)



import os
from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8", errors="ignore") as fh:
    long_description = fh.read()

def list_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join(path, filename))
    return paths

setup(
    name="mercury",
    version="1.99.8",
    maintainer="MLJAR Sp. z o.o.",
    maintainer_email="contact@mljar.com",
    description="Turn Jupyter Notebook to Web App and share with non-technical users",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=open("mercury/requirements.txt").readlines(),
    url="https://github.com/mljar/mercury",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    python_requires=">=3.7",
    entry_points={
        "console_scripts": ["mercury=mercury.mercury:main"],
    },
    package_data={"mercury": list_files("frontend-dist") + list_files("frontend-single-site-dist") + ["requirements.txt"]},
    include_package_data=True,
)



from rest_framework import serializers

from apps.tasks.models import Task


class TaskSerializer(serializers.ModelSerializer):
    class Meta:
        model = Task
        read_only_fields = ("id", "created_at", "notebook_id")
        fields = (
            "id",
            "task_id",
            "session_id",
            "created_at",
            "state",
            "params",
            "result",
        )
